-- Multi Chat V 001  by chill
-- This script may link connected hubs together by main chat

---------------
-- CHANGELOG --
---------------
-- 12.12.04 Added check for multichatflood
-- 17.12.04 Moved Profiles to Config and added commands

-- DC Funcs

dcpp:setListener( "chat","MultiChat",
	function(hub, user, data)
		local snick = user:getNick()
		MultiChat:ParseData("main", hub, snick, data)
	end
)

dcpp:setListener( "pm","MultiChat",
	function(hub, user, data)
		local snick = user:getNick()
		MultiChat:ParseData("pm", hub, snick, data)
	end
)


-- The all in one main table :)

MultiChat = {}

MultiChat._Folder = "scripts/ChillGameServer/MultiChat.Server/"
MultiChat._ConfigFile = MultiChat._Folder.."MultiChat.cfg"

MultiChat._Char = ""

MultiChat._howmany = 3 	-- How mayn multichats there are

--------------------------------------------------------------
--------------------------------------------------------------

-- Multi Configuration

MultiChat._Config = {}

MultiChat._Config._MultiChat = 1	-- 1 = Will Multi the Chat of connected hubs, 0 = will do nothing
MultiChat._Config._blockedNicks = {}	-- Table for Blocked Nicks
MultiChat._Config._blockedWords = {	-- Table for Blocked Words
	["^[%+%-%!%?]"] = 1,
}


-- Multi Hubs

MultiChat._Hubs = {}
MultiChat._HubLists = {}

-- Multi Allowed Profiles

MultiChat._Config._Profiles = {	-- By default the client may use this script in PM

	-- ["$OP"] = 1,			-- Allows all Ops to use the bot
	-- ["Chill"] = 1,		-- Allows and user with nick Chill to use the bot
}

local f = loadfile(MultiChat._ConfigFile)
if f then
	f()
end

--------------------------------------------------------------
--------------------------------------------------------------

-------------------------------------------
-- Table with chat multied chat messages --
-------------------------------------------
MultiChat._store = 5
MultiChat._Msgs = {}


function MultiChat:Main()

	for i = 1, self._howmany do
		GameServer:AddServer( "MultiChat"..i )
	end

end


function MultiChat:ParseData(mode, shub, snick, sdata)

	-- Check for Right Nick
	local multiID = self:CheckOwnNick(shub, snick)

	if multiID then

		-- do MultiChat
		MultiChat:MultiChat(mode, shub, snick, sdata, multiID)

		-- Check for Command
		local _,_,cmd = string.find( sdata, "^(%w+)")
		if cmd then
			cmd = string.lower(cmd)
			if self._Cmds[cmd] and self:AllowedProf(shub, snick) then
				self._Cmds[cmd](self, shub, snick, sdata)
				return 1
			end
		end
	end
end

function MultiChat:CheckOwnNick(shub, snick, id)
	if GameServer then
		if id then
			if GameServer:IsHub( shub, "MultiChat"..id ) then
				return 1
			end
		else
			for i = 1, self._howmany do
				if GameServer:IsHub( shub, "MultiChat"..i ) then
					return i
				end
			end
		end
	end
end

function MultiChat:MultiChat(mode, fhub, snick, data, multiID)

	if (self._Config._MultiChat == 1)
	 and self:NewMsg(data)
	 and (mode == "main")
	 and (not self._Config._blockedNicks[snick])
	 and string.find( snick, "^(%S+)$" ) then

		local ldata = string.lower(data)
		local nosend = table.foreach(self._Config._blockedWords, function(curword,_)
			if string.find(ldata,curword) then
				return 1
			end
		end)

		if not nosend then
			local hubs = dcpp:getHubs()
			table.foreach(hubs, function(_,shub)
				if (fhub ~= shub) then
					local ret = self:CheckOwnNick( shub, _, multiID )
					local ownNick = shub:getOwnNick()
					if ret and ownNick then
						DC():SendHubMessage( shub:getId(), "<"..ownNick.."> "..self._Char.." <"..snick.."> "..data.."|")
					end
				end
			end)
		end
	end
end

function MultiChat:NewMsg(data)

	local _,_,sdata = string.find(data,"([^%b<>%s])$")
	sdata = sdata or data

	local is = table.foreachi(self._Msgs, function(_,msg)
		if (msg == sdata) then
			return 1
		end
	end)

	if not is then
		table.insert(self._Msgs, 1, sdata)
		while (table.getn(self._Msgs) > self._store) do
			table.remove(self._Msgs)
		end
		return 1
	end
end


function MultiChat:AllowedProf(shub, snick)
	local ownNick = shub:getOwnNick()
	if (ownNick == snick) then
		return 1
	end
	if self._Config._Profiles then
		if self._Config._Profiles["$OP"] and shub:isOp(snick) then
			return 1
		end

		if self._Config._Profiles[snick] then
			return 1
		end
	end
end

-----------------------------
-- Multi Commands and Data --
-----------------------------

MultiChat._Data = {

	_Help =	"\t-- Multi Chat Help --\r\n\r\n"..
		"\tmultihelp    -- This text\r\n"..
		"\tmultienable   -- Enables MultiChat\r\n"..
		"\tmultidisable   -- Disables MultiChat\r\n"..
		"\tmultiblocknick <nick>  - List/Add/Del to/from MultiBlockNickList\r\n"..
		"\tmultiblockword <word>  - List/Add/Del to/from MultiBlockWordList\r\n"..
		"\tmultiprofile <prof/nick>   - List/Add/Del MultiProfile ( \"$OP\" will allow all OPs )\r\n",

}

function MultiChat:SendToNick(shub, snick, data)
	DC():SendHubMessage( shub:getId(), "$To: "..snick.." From: "..shub:getOwnNick().." $<"..shub:getOwnNick().."> "..data.."|")
end

function MultiChat:Help(shub, snick)
	self:SendToNick(shub, snick, self._Data._Help)
end

function MultiChat:Enable(shub, snick)
	self._Config._MultiChat = 1
	self:WriteTable(self._Config,"MultiChat._Config",self._ConfigFile)
	self:SendToNick(shub, snick, "MultiChat is enabled")
end

function MultiChat:Disable(shub, snick)
	self._Config._MultiChat = 0
	self:WriteTable(self._Config,"MultiChat._Config",self._ConfigFile)
	self:SendToNick(shub, snick, "MultiChat is disabled")
end

function MultiChat:AddDelBlockNick(shub, snick, sdata)
	local _,_,curnick = string.find(sdata,"%S+%s+(%S+)")
	if curnick then
		if self._Config._blockedNicks[curnick] then
			self._Config._blockedNicks[curnick] = nil
			self:WriteTable(self._Config,"MultiChat._Config",self._ConfigFile)
			self:SendToNick(shub, snick, "\""..curnick.."\" was delted from MultiBlockedNickList")
		else
			self._Config._blockedNicks[curnick] = 1
			self:WriteTable(self._Config,"MultiChat._Config",self._ConfigFile)
			self:SendToNick(shub, snick, "\""..curnick.."\" was added to MultiBlockedNickList")
		end
	else
		self:SendToNick(shub, snick, "Currently in BlockedNickList:")
		table.foreach(self._Config._blockedNicks, function(curnick,_)
			self:SendToNick(shub, snick, "  -  "..curnick)
		end)
	end
end

function MultiChat:AddDelBlockWord(shub, snick, sdata)
	local _,_,curword = string.find(sdata,"%S+%s+(%S+)")
	if curword then
		curword = string.lower( curword )
		if self._Config._blockedWords[curword] then
			self._Config._blockedWords[curword] = nil
			self:WriteTable(self._Config,"MultiChat._Config",self._ConfigFile)
			self:SendToNick(shub, snick, "\""..curword.."\" was delted from MultiBlockedWordList")
		else
			self._Config._blockedWords[curword] = 1
			self:WriteTable(self._Config,"MultiChat._Config",self._ConfigFile)
			self:SendToNick(shub, snick, "\""..curword.."\" was added to MultiBlockedWordList")
		end
	else
		self:SendToNick(shub, snick, "Currently in BlockedWordsList:")
		table.foreach(self._Config._blockedWords, function(curword,_)
			self:SendToNick(shub, snick, "  -  "..curword)
		end)
	end
end

function MultiChat:AddDelProf(shub, snick, sdata)
	local _,_,curprof = string.find(sdata,"%S+%s+(%S+)")
	if curprof then
		if self._Config._Profiles[curprof] then
			self._Config._Profiles[curprof] = nil
			self:WriteTable(self._Config,"MultiChat._Config",self._ConfigFile)
			self:SendToNick(shub, snick, "\""..curprof.."\" was delted from ProfilesList")
		else
			self._Config._Profiles[curprof] = 1
			self:WriteTable(self._Config,"MultiChat._Config",self._ConfigFile)
			self:SendToNick(shub, snick, "\""..curprof.."\" was added to ProfilesList")
		end
	else
		self:SendToNick(shub, snick, "Currently in ProfilesList:")
		table.foreach(self._Config._Profiles, function(curprof,_)
			self:SendToNick(shub, snick, "  -  "..curprof)
		end)
	end
end

MultiChat._Cmds = {

	multihelp = MultiChat.Help,			-- Show Help
	multienable = MultiChat.Enable,			-- Enable MultiChat
	multidisable = MultiChat.Disable,		-- Disable MultiChat
	multiaddbot = MultiChat.AddBot,			-- Add a bot
	multidelbot = MultiChat.DelBot,			-- Delete a bot
	multilist = MultiChat.ListBots,			-- List Bots in List
	multiblocknick = MultiChat.AddDelBlockNick,	-- List/Add/Del MultiBlockNickList
	multiblockword = MultiChat.AddDelBlockWord,	-- List/Add/Del MultiBlockWordList
	multiprofile = MultiChat.AddDelProf,		-- List/Add/Del MultiProfile

}

--------------------------------------------
function MultiChat:WriteTable(table,tablename,file)
	local handle = io.open(file,"w")
	self:Serialize(table,tablename,handle)
  	handle:close()
end
function MultiChat:Serialize(tTable,sTableName,hFile,sTab)
	sTab = sTab or "";
	hFile:write(sTab..sTableName.." = {\n");
	for key,value in tTable do
		if (type(value) ~= "function") then
			local sKey = (type(key) == "string") and string.format("[%q]",key) or string.format("[%d]",key);
			if(type(value) == "table") then
				self:Serialize(value,sKey,hFile,sTab.."\t");
			else
				local sValue = (type(value) == "string") and string.format("%q",value) or tostring(value);
				hFile:write(sTab.."\t"..sKey.." = "..sValue);
			end
			hFile:write(",\n");
		end
	end
	hFile:write(sTab.."}");
end
--------------------------------------------

MultiChat:Main()

----------------------------
----------------------------
DC():PrintDebug( "  ** Loaded MultiChat.lua **" )